package com.nisum.ascend.promotions.service;

import com.nisum.ascend.promotions.dto.PromotionDto;
import com.nisum.ascend.promotions.exception.ResourceNotFoundException;
import com.nisum.ascend.promotions.model.Promotion;
import com.nisum.ascend.promotions.repository.PromotionRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import java.util.List;
import reactor.core.publisher.Mono;

@Service
public class PromotionService {

    @Autowired
    PromotionRepository promotionRepository;

    public Flux<PromotionDto> findAll(){
        return promotionRepository.findAll().map(PromotionDto::generateDtoFromPromotion);
    }

    public Mono<PromotionDto> findPromoById(String promoId) {
        return promotionRepository.findByPromotionId(promoId)
                .map(PromotionDto::generateDtoFromPromotion);
    }

    public Flux<PromotionDto> findPromotionsByProductSku(String sku){
        return promotionRepository.findByProductSku(sku).map(PromotionDto::generateDtoFromPromotion);
    }

    public Flux<PromotionDto> bulkSearchSku(List<String> skus){
        return Flux.fromIterable(skus).flatMap(sku -> findPromotionsByProductSku(sku));
    }

    public Mono<PromotionDto> createPromotion(Promotion promotion){
        return promotionRepository.save(promotion).map(PromotionDto::generateDtoFromPromotion);
    }
    public Mono<PromotionDto> updatePromotion(String promoId, Promotion promotion){
        return promotionRepository.findByPromotionId(promoId)
                .flatMap(promo ->{
                    promo.setDiscountPercentage(promotion.getDiscountPercentage());
                    promo.setProductSku(promotion.getProductSku());
                    promo.setMinimumQuantity(promotion.getMinimumQuantity());
                    return promotionRepository.save(promo).map(PromotionDto::generateDtoFromPromotion);
                });
    }

    public Mono<Promotion> deletePromotion(String promoId){
        return promotionRepository.findByPromotionId(promoId)
                .flatMap(promo -> promotionRepository.delete(promo)
                        .then(Mono.just(promo)))
                        .switchIfEmpty(Mono.error(new ResourceNotFoundException(HttpStatus.NOT_FOUND, "Promo Not Found")));


    }
}
